------------------------------------------
---  Copyright HDRsoft - www.hdrsoft.com   
------------------------------------------

local LrApplication = import 'LrApplication'
local LrBinding = import 'LrBinding'
local LrView = import 'LrView'
local LrDialogs = import 'LrDialogs'
local LrFunctionContext = import 'LrFunctionContext'
local LrErrors = import 'LrErrors'
local LrFileUtils = import 'LrFileUtils'
local LrPathUtils = import 'LrPathUtils'
local LrLogger = import 'LrLogger'
local prefs = import 'LrPrefs'.prefsForPlugin() 
local LrTasks = import 'LrTasks'

local myLogger = LrLogger('PhotomatixLogger')
myLogger:disable("print") 


local cntPhotos=0
local cntNeedUpgrade=0
local cntUpgraded=0
local folderPath=''

Upgrade32bitTiff = {}



function GetAppPath ( )
	local appPath=LrPathUtils.child(_PLUGIN.path, "app")

	local clProgName="Merge32Utils"
	if (MAC_ENV) then
		clProgName="MergeToHDR"
	end

	appPath=LrPathUtils.child(appPath, clProgName)
	
	if (WIN_ENV) then
		appPath=LrPathUtils.addExtension(appPath, "exe")
	end
	
	if not LrFileUtils.exists(appPath) then
		return false  -- Cannot tell
	end

	return appPath
end



function CheckOrUpdateLegacy32bitTiff(checkOrUpdateType, fileName)

	myLogger:trace(string.format('CheckUpdateType: %s  fileName: %s', checkOrUpdateType, fileName))

	local appPath=GetAppPath()
	
	if not appPath then
		return false
	end

	local cmd
	if (WIN_ENV) then
		cmd = string.format('\"%s', appPath, fileName)
	else
		cmd=string.format('\"%s\"', appPath, fileName)
	end


	if (checkOrUpdateType=='isFP') then
		cmd=string.format('%s -tf', cmd)
	elseif (checkOrUpdateType=='isLegacy') then
		cmd=string.format('%s -uc', cmd)
	elseif (checkOrUpdateType=='upgradeLegacy') then
		cmd=string.format('%s -uu', cmd)	
	end
	
	if (WIN_ENV) then
		cmd=string.format('%s \"%s\"\"', cmd, fileName)
	else
		cmd=string.format('%s \"%s\"', cmd, fileName)
	end

	
	myLogger:trace(string.format('Running command: %s',cmd))
	
	retour=LrTasks.execute(cmd)
	myLogger:trace(string.format('Returned: %s',retour))
	
	local ReturnedValWhenValidated=1
	if (MAC_ENV) then
		ReturnedValWhenValidated=0
	end

	if retour == ReturnedValWhenValidated then
		return true
	else
		return false
	end
end



function CheckOrUpgradeSelectedPhotos(isCheckOrUpgrade)
	local checkStatus='ok' 
	local strErrMessage=''
	local iPhoto, wFirstPhoto, hFirstPhoto=0
	local activeCatalog = LrApplication.activeCatalog()

	activeCatalog:withReadAccessDo( function()

		local photos = activeCatalog.targetPhotos	
		cntPhotos=#photos

		for iPhoto=1, cntPhotos, 1 do		
			local wImage = photos[iPhoto]:getRawMetadata('width')
			local hImage = photos[iPhoto]:getRawMetadata('height')
			local fileSize = photos[iPhoto]:getRawMetadata('fileSize')
			local fileFormat = photos[iPhoto]:getRawMetadata('fileFormat')
			local fileName=photos[iPhoto]:getFormattedMetadata('fileName')
			local fullPath = photos[iPhoto]:getRawMetadata('path')				
			
			local doCheck=true
			if not wImage or not hImage or not fileSize or not fileFormat or (wImage<=0) or (hImage<=0) then
				doCheck=false
			else 
				myLogger:trace(string.format('%d x %d  - format: %s -  fileSize: %d  is  %1.2f  times resolution',wImage,hImage,fileFormat,fileSize,fileSize/(wImage*hImage)))	
			end

			if ( doCheck and (fileFormat=='TIFF') ) then
				
				-- Checking whether Floating Point TIFF
				local isFloatingPointTiff=false
				if (fileSize>=(wImage*hImage*11))  then  -- 32-bit TIFF
					isFloatingPointTiff=true
				elseif (fileSize >= (wImage*hImage*5)) then  -- 16-bit TIFF
					isFloatingPointTiff=CheckOrUpdateLegacy32bitTiff('isFP', fullPath)
				end
				
				if isFloatingPointTiff then
					if isCheckOrUpgrade=='check' then
						if CheckOrUpdateLegacy32bitTiff('isLegacy', fullPath) then										
							cntNeedUpgrade=cntNeedUpgrade+1
							myLogger:trace(string.format('+++++++++++++  Found legacy file:  %s  -  total found: %d',fileName,cntNeedUpgrade))	
						end
					elseif isCheckOrUpgrade=='upgrade' then
						if CheckOrUpdateLegacy32bitTiff('upgradeLegacy', fullPath) then										
							cntUpgraded=cntUpgraded+1
						end
					end
				end		
				
			end
			
		end
	end)
	return checkStatus
end



local function showUpgradeDialog( )

	LrFunctionContext.callWithContext( 'Upgrade32bitTiff', function( context )

	local f = LrView.osFactory() 
	local props = LrBinding.makePropertyTable( context ) 

	local upgradeCheck = f:column {
		bind_to_object = props,
		spacing = f:dialog_spacing(),

		f:row {
			f:static_text {
				title = "Clicking on the \"Check And Upgrade\" button will do the following:",
				alignment = 'left',
			},
		},

		f:row {
			f:static_text {
				title = "- Check which of the files you selected are Floating Point TIFF files \n  created by version 1.0 of the Merge to 32-bit HDR Plug-in.",
				alignment = 'left',
			},
		},

		f:row {
			f:static_text {
				title = "- Ask to confirm you want to upgrade the files to make them compatible\n  with Lightroom 4.4, and if so proceed with upgrading.",
				alignment = 'left',
			},
		},
	}
	

	local result = LrDialogs.presentModalDialog( {
		title = LOC "Upgrade 32-bit TIFF files",
		contents = upgradeCheck,
		actionVerb="Check And Upgrade"
	} )	
	

	if (result=='ok') then

		LrTasks.startAsyncTask( function()
					
	    cntNeedUpgrade=0
	
		local validated=CheckOrUpgradeSelectedPhotos('check')

						if validated then
							if ( cntNeedUpgrade > 0) then

								local confMessage=string.format("Found %d Floating Point TIFF files that need to be upgraded among the %d files selected.\n\nAre you sure you want to upgrade the %d Floating Point TIFF files to make them compatible with Lightroom v4.4?",cntNeedUpgrade,cntPhotos,cntNeedUpgrade)
	
								if (cntNeedUpgrade==1) then
									if (cntPhotos==1) then
										confMessage="The file selected is a Floating Point TIFF files that needs to be upgraded to be compatible with Lightroom 4.4.\n\nAre you sure you want to upgrade the file?" 									
									else
										confMessage=string.format("Found one Floating Point TIFF file that needs to be upgraded among the %d files selected.\n\nAre you sure you want to upgrade the Floating Point TIFF file to make it compatible with Lightroom v4.4?",cntPhotos)
									end
								
								end
								
								local userActionConf = LrDialogs.confirm( "Confirm request for upgrade",confMessage, "Upgrade Files", "Cancel")
									
								if ( (userActionConf~=nil) and (userActionConf=='ok') ) then
									cntUpgraded=0
									validated=CheckOrUpgradeSelectedPhotos('upgrade')
									LrDialogs.message("Upgrade successful", string.format("Upgraded %d Floating Point TIFF file(s) for compatibility with Lightroom 4.4.",cntUpgraded),"info")
								end
								
							else
								LrDialogs.message("No upgrade needed", string.format("Did not find any Floating Point TIFF file needing to be upgraded among the %d files selected.",cntPhotos),"info")
							end

						else
							LrDialogs.message("Error when checking files")
							
						end	
					end)
					

end


end )

end	
			
showUpgradeDialog( )
